#ifndef _IMAGETOOLS_H
#define _IMAGETOOLS_H
#include "image.H"
/** @name Outils Image
    @memo Fichier regroupant un ensemble d'outils utiles pour la manipulation d'images.
    @doc {\bf Fichier a inclure :}
    \begin{verbatim}
    #include "imagetools.H"
    \end{verbatim}
    \\
    {\bf Exemple :}
    \begin{verbatim}
    #include "image.H"
    #include "imageio.H"
    #include "imagetools.H"

    int main (void)
    {
       // lecture image PGM
       Image<octet> im;
       ImageIO<octet>::ReadPGM("toto.pgm",im);

       // lecture PPM
       Image<RGB> im2;
       ImageIO<RGB>::ReadPPM("toto.ppm",im2);

       // transfo en YUV
       float *y,*u,*v;
       int h,w;
       ImageTools<RGB>::RGBtoYUV(im2,y,u,v,h,w);

       // transfo du Y en image d octet
       Image<float> imf(h,w,y);
       Image<octet> imo(ImageTools<float>::convertTo8bits(imf));

       // calcul du max
       cout<<"Max="<<(int)ImageTools<octet>::getMax(imo)<<endl;

       // calcul du PSNR entre l original et la composante Y
       cout<<"PSNR="<<ImageTools<octet>::getPSNR(im,imo)<<endl;

       delete []y;
       delete []u;
       delete []v;

       }
       \end{verbatim}
    @author \URL[Thomas Guionnet]{mailto:Thomas.Guionnet@irisa.fr}
    @author \URL[Stephane Pateux]{mailto:Stephane.Pateux@irisa.fr}
    @author \URL[Franck Galpin]{mailto:Franck.Galpin@irisa.fr}
    @see Image ImageIO
*/
//@{

/** @name Type enumeres pour la conversion */
//@{
/** Type de conversion lors du passage T vers octet (utile dans convertTo8bits et functionHisto).\\
    @doc Valeurs possibles :
    \begin{itemize}
    \item NORMAL rescale des valeurs entre 0 et 255
    \item LN1 rescale des valeurs entre 0 et 255 sur une echelle logarithmique
    \item VAR1 rescale des valeurs entre -param*ecart-type et +param*ecart-type
    \item SCALE rescale avec le facteur param
    \item VAR2 rescale idem a VAR1 pour des valeurs positives
    \item EGAL : egalise l'histogramme,
    \item EXP : rend l'histo exponentielle (privilegie les valeurs faibles)
    \item LOG : rend l'histo logarithmique (privilegie les valeurs hautes)
    \item EXPINV : rend l'histo exponentielle inverse (privilegie les valeurs hautes)
    \item LOGINV : rend l'histo logarithmique inverse (privilegie les valeurs faibles)
    \item SIN : rend l'histo sinus (privilegie les valeurs extremes)
    \item COS : rend l'histo cossinus (privilegie les valeurs moyennes)
    \end{itemize} */
enum TypeConv8bits { NORMAL, LN1, VAR1,SCALE,VAR2, EGAL, EXP, LOG, EXPINV,LOGINV, SIN, COS };

/** Type de transformation YUV possible.\\
 @doc Valeurs possibles :
    \begin{itemize}
    \item NOSPEC format non specifie
    \item ITU709 format ITU-709
    \item UNSPEC format non specifie
    \item RESERVED reserve
    \item FCC format fcc
    \item ITU6244 format ITU-6244
    \item SMPTE170M format SMPTE-170M
    \item SMPTE240M format SMPTE-240M
    \end{itemize} */
enum YUVTransfo { NOSPEC=0, ITU709, UNSPEC, RESERVED, FCC, ITU6244, SMPTE170M, SMPTE240M };

/** Taille des composante YUV lors des conversion RGB-YUV.\\
  @doc Valeurs possibles :
  \begin{itemize}
  \item S420 U et V decimes par 2 en hauteur et largeur
  \item S422 U et V decimes par 2 en hauteur (entrelace)
  \item S444 pleine resolution
  \end{itemize}
*/
enum YUVSize { S420=0, S422, S444};
//@}

/** @name Definition de type simple */
//@{
/** Definition simple du type octet. */
#define octet unsigned char

/** Classe de pixel couleur. */
class RGB {
public:
  /** @doc constructeur par defaut du type RGB */
  RGB():R(0),G(0),B(0) {};

  /** @doc constructeur avec initialisation du type RGB
      @param r valeur de la composante R
      @param g valeur de la composante G
      @param b valeur de la composante B */
  RGB(octet r,octet g,octet b):R(r),G(g),B(b) {};

  /** @doc constructeur avec initialisation du type RGB
      @param gris valeur des composantes R G et B */
  RGB(octet gris):R(gris),G(gris),B(gris) {};

  /** @doc operateur de recopie
      @param pix pixel RGB a recopier
      @return une reference sur le pixel */
  inline RGB & operator=(const RGB &pix) { R=pix.R; G=pix.G; B=pix.B; return(*this); };

  /** composante R */
  octet R;

  /** composante G */
  octet G;

  /** composante B */
  octet B;
};
//@}

/** Classe regroupant des fonctions utiles a la manipulation d'image. */
template <class T>
class ImageTools {
public:

  /** @name Fontions usuelles  */
  //@{
  /** @doc permet d'obtenir une valeur interpixel par interpolation bilineaire (le type T doit etre compatible avec la multiplication par un double)
      @param image image a interpoler
      @param yy ordonnee (numero de ligne) du pixel a intepoler
      @param xx abscisse (numero de colonne) du pixel a intepoler
      @return la valeur interpixel interpolee */
  static T bilinear(const Image<T> &image,const double yy,const double xx);

  /** @doc permet d'obtenir la valeur maximum d'une image
      @param image image en entree
      @return la valeur maximale trouvee (valeur par  defaut si image de taille nulle) */
  static T getMax(const Image<T> &image);

  /** @doc permet d'obtenir le PSNR entre 2 images (le type T doit etre compatible avec l'octet)
      @param image1 image1 en entree
      @param image2 image2 en entree
      @return la valeur du PSNR entre les 2 images */
  static double getPSNR(const Image<T> &image1,const Image<T> &image2);

  /** @doc permet de compacter une image d'etiquette (le type T doit etrecompatible avec un int)
      @param image image en entree */
  static void compacte(Image<T> &image);

  /** @doc permet de dilater une image en 8-connexite (les valeurs doivent etre 0 ou 1)
      @param in_image image en entree
      @param taille de l'element structurant */
  static void dilate(Image<T> &in_image,const int taille);

  /** @doc permet de convertir une image de type T en image d'octet
      @param image1 image a convertir
      @param type type de conversion (NORMAL: scale, LN1:scale logarithmique, VAR1:scale centre normalise)
      @param param pour type=VAR1 : choisit le nombre d'ecart-type autour de la moyenne
      @return image d'octet convertie */
  static Image<octet> convertTo8bits(const Image<T> &image1,const TypeConv8bits type=NORMAL,const double param=1.);

  /** @doc permet de convertir une image de type T en image d'octet par fonctionnalisation d'histogramme
      @param image1 image a convertir
      @param type type de conversion (EGAL: egalisation de l'histogramme, EXP:rend l'histogramme exp, LOG: rend l'histogramme log, EXPINV : rend l'histogramme exp inv, LOGINV: rend l'histogramme log inv, SIN: rend l'histogramme en sinus, COS: rend l'histogramme en cosinus)
      @return image d'octet convertie */
  static Image<octet> functionHisto(const Image<T> &image1,const TypeConv8bits type=EGAL);

  /** @doc permet de convertir une image RGB en YUV (ATTENTION : n'existe que pour le type T=RGB)
      @param ima image RGB a convertir
      @param imay image de flottants de la composante Y
      @param imau image de flottants de la composante U
      @param imav image de flottants de la composante V
      @param h hauteur de l'image Y
      @param w largeur de l'image Y
      @param size valeur possible : S420, S422, S444
      @param typetransfo valeur possible : NOSPEC, ITU709, UNSPEC, RESERVED, FCC, ITU6244, SMPTE170M, SMPTE240M */
  static void RGBtoYUV(const Image<T> &ima, float *&imay,float *&imau,float *&imav,int &h,int &w,YUVSize size=S420,YUVTransfo typetransfo=SMPTE170M) {exit(-1);}; // vide par defaut

  /** @doc permet de convertir une image YUV en RGB (ATTENTION : n'existe que pour le type T=RGB)
      @param imay image de flottants de la composante Y
      @param imau image de flottants de la composante U
      @param imav image de flottants de la composante V
      @param h hauteur de l'image Y
      @param w largeur de l'image Y
      @param typetransfo valeur possible : NOSPEC, ITU709, UNSPEC, RESERVED, FCC, ITU6244, SMPTE170M, SMPTE240M
      @param size valeur possible : S420, S422, S444 */
  static Image<T> YUVtoRGB(const float *const imay,const float *const imau,const float *const imav,
			   const int h,const int w,
			   YUVTransfo typetransfo=SMPTE170M,YUVSize size=S420) {
    exit(-1);
    return(Image<T>(0,0));
  };
  //@}
};
//@}

#include "imagetools.cpp"

#endif
